# vim:tw=0:expandtab:autoindent:tabstop=4:shiftwidth=4:filetype=python:

  #############################################################################
  #
  # Copyright (c) 2005 Dell Computer Corporation
  # Dual Licenced under GNU GPL and OSL
  #
  #############################################################################
"""
systeminfo:
    python interface to functions in libsmbios_c  system_info.h
"""

# imports (alphabetical)
import ctypes

from libsmbios_c import libsmbios_c_DLL as DLL
from ._common import errorOnNullPtrFN, errorOnNegativeFN, errorOnZeroFN, freeLibStringFN, c_utf8_p
from .trace_decorator import traceLog, getLog, strip_trailing_whitespace

__all__ = []

#// format error string
#const char *token_table_strerror(const struct token_table *m);
# define strerror first so we can use it in error checking other functions.
DLL.sysinfo_strerror.argtypes = [ ]
DLL.sysinfo_strerror.restype = c_utf8_p
@traceLog()
def _strerror():
    print(DLL.sysinfo_strerror())
    return Exception

#    const char *smbios_get_library_version_string();
DLL.smbios_get_library_version_string.argtypes = []
DLL.smbios_get_library_version_string.restype = c_utf8_p
get_library_version_string = DLL.smbios_get_library_version_string
__all__.append("get_library_version_string")

#    const char *smbios_get_library_version_major();
DLL.smbios_get_library_version_major.argtypes = []
DLL.smbios_get_library_version_major.restype = ctypes.c_int
get_library_version_major = DLL.smbios_get_library_version_major
__all__.append("get_library_version_major")

#    const char *smbios_get_library_version_minor();
DLL.smbios_get_library_version_minor.argtypes = []
DLL.smbios_get_library_version_minor.restype = ctypes.c_int
get_library_version_minor = DLL.smbios_get_library_version_minor
__all__.append("get_library_version_minor")

#    const char *sysinfo_get_dell_system_id();
DLL.sysinfo_get_dell_system_id.argtypes = []
DLL.sysinfo_get_dell_system_id.restype = ctypes.c_int
DLL.sysinfo_get_dell_system_id.errcheck=errorOnZeroFN(lambda r,f,a: Exception(_("Could not determine System ID.")))
get_dell_system_id = DLL.sysinfo_get_dell_system_id
__all__.append("get_dell_system_id")

#    void sysinfo_string_free( const char * );
DLL.sysinfo_string_free.argtypes = [ctypes.POINTER(ctypes.c_char),]
DLL.sysinfo_string_free.restype = None

def _mk_simple_sysinfo_str_fn(name):
    import sys
    getattr(DLL,  "sysinfo_%s" % name).argtypes=[]
    #getattr(DLL,  "sysinfo_%s" % name).restype=c_utf8
    getattr(DLL,  "sysinfo_%s" % name).restype=ctypes.POINTER(ctypes.c_char)
    getattr(DLL,  "sysinfo_%s" % name).errcheck=strip_trailing_whitespace()(freeLibStringFN( DLL.sysinfo_string_free, lambda r,f,a: Exception(_strerror() )))
    sys.modules[__name__].__dict__[name] = getattr(DLL,  "sysinfo_%s" % name)
    __all__.append(name)

_mk_simple_sysinfo_str_fn("get_system_name")
_mk_simple_sysinfo_str_fn("get_vendor_name")
_mk_simple_sysinfo_str_fn("get_bios_version")
_mk_simple_sysinfo_str_fn("get_asset_tag")
_mk_simple_sysinfo_str_fn("get_service_tag")
_mk_simple_sysinfo_str_fn("get_property_ownership_tag")

@traceLog()
def set_service_tag(newtag, pass_ascii=None, pass_scancode=None):
    return DLL.sysinfo_set_service_tag(newtag, pass_ascii, pass_scancode)
__all__.append("set_service_tag")

DLL.sysinfo_set_asset_tag.argtypes = [ctypes.c_char_p, ctypes.c_char_p, ctypes.c_char_p]
DLL.sysinfo_set_asset_tag.restype = ctypes.c_int
DLL.sysinfo_set_asset_tag.errcheck=errorOnNegativeFN(lambda r,f,a: _strerror())
@traceLog()
def set_asset_tag(newtag, pass_ascii=None, pass_scancode=None):
    return DLL.sysinfo_set_asset_tag(newtag, pass_ascii, pass_scancode)
__all__.append("set_asset_tag")

DLL.sysinfo_set_property_ownership_tag.argtypes = [ctypes.c_char_p, ctypes.c_char_p, ctypes.c_char_p]
DLL.sysinfo_set_property_ownership_tag.restype = ctypes.c_int
DLL.sysinfo_set_property_ownership_tag.errcheck=errorOnNegativeFN(lambda r,f,a: _strerror())
@traceLog()
def set_property_ownership_tag(newtag, pass_ascii=None, pass_scancode=None):
    return DLL.sysinfo_set_property_ownership_tag(newtag, pass_ascii, pass_scancode)
__all__.append("set_property_ownership_tag")

if __name__ == "__main__":
    exitRet = 0
    def pr(s, f):
        try:
            ret = f()
        except Exception as e:
            exitRet=1
            ret = str(e)
            print("Error getting the", end=' ')

        print(s % ret)

    pr("Libsmbios:    %s", get_library_version_string)

    try:
        sysid = "0x%04X" % get_dell_system_id()
    except Exception as e:
        exitRet=1
        print("Error getting the", end=' ')
        sysid = str(e)
    print("System ID:    %s" % sysid)

    pr("Service Tag:  %s", get_service_tag)
    pr("Asset Tag:    %s", get_asset_tag)

    try:
        esc = int(get_service_tag(), 36)
    except Exception as e:
        esc = str(e)
        print("Error getting the", end=' ')

    print("Express Service Code: %s" % esc)
    pr("Product Name: %s", get_system_name)
    pr("BIOS Version: %s", get_bios_version)
    pr("Vendor:       %s", get_vendor_name)
    pr("Property Ownership Tag: %s", get_property_ownership_tag)

    import sys
    sys.exit(exitRet)




# purpose of the following is so we can easily run from built source tree if
# topdir == builddir. (ie. doesnt support out of tree builds, set
# LD_LIBRARY_PATH manually for that)
import os
from ._vars import *

# import the .so one time rather than in each submodule
import ctypes
try:
    # try explicit path first
    # this lets us run cleanly from build tree
    # or if user uses wierd --prefix for configure
    # doesnt work from build tree if top_builddir != top_srcdir
    libsmbios_c_DLL = ctypes.cdll.LoadLibrary(os.path.join(libdir, LIBSMBIOS_C_SONAME))
except OSError as e:
    # then let it search
    libsmbios_c_DLL = ctypes.cdll.LoadLibrary(LIBSMBIOS_C_SONAME)

# import all our packages.
from . import cmos
from . import memory
from . import smbios
from . import smi
from . import system_info
from . import smbios_token
from . import _common

import gettext
gettext.install(GETTEXT_PACKAGE, localedir)

__VERSION__ = "uninstalled-version"

_all_ = [
    "cmos", "memory", "smbios", "smi", "system_info", "smbios_token",
    "GETTEXT_PACKAGE",
    "localedir", "pkgdatadir", "pythondir", "pkgconfdir"
    ]



# vim:tw=0:expandtab:autoindent:tabstop=4:shiftwidth=4:filetype=python:

  #############################################################################
  #
  # Copyright (c) 2005 Dell Computer Corporation
  # Dual Licenced under GNU GPL and OSL
  #
  #############################################################################
"""
memory_obj:
    python interface to functions in libsmbios_c  obj/memory.h
"""

# imports (alphabetical)
import ctypes

from libsmbios_c import libsmbios_c_DLL as DLL
from ._common import errorOnNegativeFN, errorOnNullPtrFN, c_utf8_p
from .trace_decorator import traceLog, getLog

__all__ = ["MemoryAccess", "MEMORY_DEFAULTS", "MEMORY_GET_SINGLETON", "MEMORY_GET_NEW", "MEMORY_UNIT_TEST_MODE"]

MEMORY_DEFAULTS      =0x0000
MEMORY_GET_SINGLETON =0x0001
MEMORY_GET_NEW       =0x0002
MEMORY_UNIT_TEST_MODE=0x0004

@traceLog()
def MemoryAccess(flags=MEMORY_GET_SINGLETON, *factory_args):
    if flags & MEMORY_GET_SINGLETON:
        if _MemoryAccess._instance is None:
            _MemoryAccess._instance = _MemoryAccess( flags, *factory_args)
        return _MemoryAccess._instance
    else:
        return _MemoryAccess( flags, *factory_args)

class _MemoryAccess(ctypes.Structure):
    _instance = None

    @traceLog()
    def __init__(self, *args):
        self._memobj = None
        self._memobj = DLL.memory_obj_factory(*args)

    # dont decorate __del__
    def __del__(self):
        DLL.memory_obj_free(self._memobj)

    @traceLog()
    def read(self, offset, length):
        buf = ctypes.create_string_buffer(length)
        DLL.memory_obj_read(self._memobj, buf, offset, length)
        return buf

    @traceLog()
    def write(self, buf, offset):
        DLL.memory_obj_write(self._memobj, buf, offset, len(buf))

    @traceLog()
    def search(self, pattern, start, end, stride):
        return DLL.memory_obj_search(self._memobj, pattern, len(pattern), start, end, stride)

    @traceLog()
    def close_hint(self, hint=None):
        if hint is not None:
            if hint:
                DLL.memory_obj_suggest_leave_open(self._memobj)
            else:
                DLL.memory_obj_suggest_close(self._memobj)

        return DLL.memory_obj_should_close(self._memobj)


# define strerror first so we can use it in error checking other functions.
DLL.memory_obj_strerror.argtypes = [ ctypes.POINTER(_MemoryAccess) ]
DLL.memory_obj_strerror.restype = c_utf8_p
@traceLog()
def _strerror(obj):
    return Exception(DLL.memory_obj_strerror(obj))

#struct memory_access_obj *memory_obj_factory(int flags, ...);
# dont define argtypes because this is a varargs function...
#DLL.memory_obj_factory.argtypes = [ctypes.c_int, ]
DLL.memory_obj_factory.restype = ctypes.POINTER(_MemoryAccess)
DLL.memory_obj_factory.errcheck = errorOnNullPtrFN(lambda r,f,a: _strerror(r))

#void memory_obj_free(struct memory_access_obj *);
DLL.memory_obj_free.argtypes = [ ctypes.POINTER(_MemoryAccess) ]
DLL.memory_obj_free.restype = None

#int  memory_obj_read(const struct memory_access_obj *, void *buffer, u64 offset, size_t length);
DLL.memory_obj_read.argtypes = [ ctypes.POINTER(_MemoryAccess), ctypes.c_void_p, ctypes.c_uint64, ctypes.c_size_t ]
DLL.memory_obj_read.restype = ctypes.c_int
DLL.memory_obj_read.errcheck = errorOnNegativeFN(lambda r,f,a: _strerror(a[0]))

#int  memory_obj_write(const struct memory_access_obj *, void *buffer, u64 offset, size_t length);
DLL.memory_obj_write.argtypes = [ ctypes.POINTER(_MemoryAccess), ctypes.c_void_p, ctypes.c_uint64, ctypes.c_size_t ]
DLL.memory_obj_write.restype = ctypes.c_int
DLL.memory_obj_write.errcheck = errorOnNegativeFN(lambda r,f,a: _strerror(a[0]))

#s64  memory_obj_search(const struct memory_access_obj *, const char *pat, size_t patlen, u64 start, u64 end, u64 stride);
DLL.memory_obj_search.argtypes = [ ctypes.POINTER(_MemoryAccess), ctypes.c_char_p, ctypes.c_size_t, ctypes.c_uint64, ctypes.c_uint64, ctypes.c_uint64 ]
DLL.memory_obj_search.restype = ctypes.c_int64
DLL.memory_obj_search.errcheck = errorOnNegativeFN(lambda r,f,a: _strerror(a[0]))

#void  memory_obj_suggest_leave_open(struct memory_access_obj *);
DLL.memory_obj_suggest_leave_open.argtypes = [ ctypes.POINTER(_MemoryAccess), ]
DLL.memory_obj_suggest_leave_open.restype = None

#void  memory_obj_suggest_close(struct memory_access_obj *);
DLL.memory_obj_suggest_close.argtypes = [ ctypes.POINTER(_MemoryAccess), ]
DLL.memory_obj_suggest_close.restype = None

#bool  memory_obj_should_close(const struct memory_access_obj *);
DLL.memory_obj_should_close.argtypes = [ ctypes.POINTER(_MemoryAccess), ]
DLL.memory_obj_should_close.restype = ctypes.c_bool



# vim:tw=0:expandtab:autoindent:tabstop=4:shiftwidth=4:filetype=python:

  #############################################################################
  #
  # Copyright (c) 2005 Dell Computer Corporation
  # Dual Licenced under GNU GPL and OSL
  #
  #############################################################################
"""
_common:

"""

# imports (alphabetical)
import ctypes

from .trace_decorator import traceLog, getLog

__all__ = ["freeLibStringFN", "errorOnZeroFN", "errorOnNegativeFN", "errorOnNullPtrFN" ]

class SMBIOSException(Exception): pass

def _doExc(exception_fn, r, f, a, msg):
    if exception_fn is not None:
        raise exception_fn(r, f, a)
    else:
        raise SMBIOSException( msg )

def freeLibStringFN(free_fn, exception_fn=None):
    @traceLog()
    def _freeLibStringFN(result, func, args):
        getLog(prefix="trace.").info("RAN CTYPES FUNCTION: %s" % func.__name__)
        pystr = ctypes.cast(result, ctypes.c_char_p).value
        if pystr is None:
            pystr = ""
            #_doExc(exception_fn, result, func, args, _("null string returned") )

        free_fn(result)
        return pystr
    return _freeLibStringFN

def errorOnNullPtrFN(exception_fn=None):
    @traceLog()
    def _errorOnNullPtrFN(result, func, args):
        getLog(prefix="trace.").info("RAN CTYPES FUNCTION: %s" % func.__name__)
        if not bool(result): # check for null pointer
            _doExc(exception_fn, result, func, args, _("null pointer returned") )
        return result
    return _errorOnNullPtrFN

def errorOnZeroFN(exception_fn=None):
    @traceLog()
    def _errorOnZeroFN(result, func, args):
        getLog(prefix="trace.").info("RAN CTYPES FUNCTION: %s" % func.__name__)
        if result is None or result == 0:
            _doExc(exception_fn, result, func, args, _("function returned error value of zero") )
        return result
    return _errorOnZeroFN

def errorOnNegativeFN(exception_fn=None):
    @traceLog()
    def _errorOnNegativeFN(result, func, args):
        getLog(prefix="trace.").info("RAN CTYPES FUNCTION: %s" % func.__name__)
        if result is None or result < 0:
            _doExc(exception_fn, result, func, args, _("function returned negative error code") )
        return result
    return _errorOnNegativeFN

class c_utf8_p(ctypes.c_char_p):
    @classmethod
    def _check_retval_(cls, result):
        value = result.value
        try:
            return value.decode('utf-8')
        except:
            return value



# vim:tw=0:expandtab:autoindent:tabstop=4:shiftwidth=4:filetype=python:

  #############################################################################
  #
  # Copyright (c) 2005 Dell Computer Corporation
  # Dual Licenced under GNU GPL and OSL
  #
  #############################################################################
"""
cmos_obj:
    python interface to functions in libsmbios_c  obj/cmos.h
"""

# imports (alphabetical)
import ctypes

from libsmbios_c import libsmbios_c_DLL as DLL
from ._common import errorOnNegativeFN, errorOnNullPtrFN, c_utf8_p
from .trace_decorator import traceLog

__all__ = ["CmosAccess", "CMOS_DEFAULTS", "CMOS_GET_SINGLETON", "CMOS_GET_NEW", "CMOS_UNIT_TEST_MODE"]

CMOS_DEFAULTS      =0x0000
CMOS_GET_SINGLETON =0x0001
CMOS_GET_NEW       =0x0002
CMOS_UNIT_TEST_MODE=0x0004

@traceLog()
def CmosAccess(flags=CMOS_GET_SINGLETON, *factory_args):
    if flags & CMOS_GET_SINGLETON:
        if _CmosAccess._instance is None:
            _CmosAccess._instance = _CmosAccess( flags, *factory_args)
        return _CmosAccess._instance
    else:
        return _CmosAccess( flags, *factory_args)

class _CmosAccess(ctypes.Structure):
    _instance = None

    @traceLog()
    def __init__(self, *args):
        self._cmosobj = None
        self._cmosobj = DLL.cmos_obj_factory(*args)
        self._callbacks = []

    # dont decorate __del__
    def __del__(self):
        DLL.cmos_obj_free(self._cmosobj)

    @traceLog()
    def readByte(self, indexPort, dataPort, offset):
        buf = ctypes.c_uint8()
        DLL.cmos_obj_read_byte(self._cmosobj, buf, indexPort, dataPort, offset)
        return buf.value

    @traceLog()
    def writeByte(self, buf, indexPort, dataPort, offset):
        DLL.cmos_obj_write_byte(self._cmosobj, buf, indexPort, dataPort, offset)

    @traceLog()
    def registerCallback(self, callback, userdata, freecb):
        cb = WRITE_CALLBACK(callback)
        # append callback to array that has same lifetime as object so python
        # doesnt garbage collect it from under us
        self._callbacks.append(cb)

        # special handling for possible NULL free callback
        fcb = ctypes.cast(None, FREE_CALLBACK)
        if freecb is not None:
            fcb = FREE_CALLBACK(freecb)
            self._callbacks.append(fcb)

        DLL.cmos_obj_register_write_callback(self._cmosobj, cb, userdata, fcb)


#// format error string
#const char *cmos_obj_strerror(const struct cmos_access_obj *m);
# define strerror first so we can use it in error checking other functions.
DLL.cmos_obj_strerror.argtypes = [ ctypes.POINTER(_CmosAccess) ]
DLL.cmos_obj_strerror.restype = c_utf8_p
@traceLog()
def _strerror(obj):
    return Exception(DLL.cmos_obj_strerror(obj))

#struct cmos_access_obj *cmos_obj_factory(int flags, ...);
# dont define argtypes because this is a varargs function...
#DLL.cmos_obj_factory.argtypes = [ctypes.c_int, ]
DLL.cmos_obj_factory.restype = ctypes.POINTER(_CmosAccess)
DLL.cmos_obj_factory.errcheck = errorOnNullPtrFN(lambda r,f,a: _strerror(r))

#void   cmos_obj_free(struct cmos_access_obj *);
DLL.cmos_obj_free.argtypes = [ ctypes.POINTER(_CmosAccess) ]
DLL.cmos_obj_free.restype = None

#int     cmos_obj_read_byte(const struct cmos_access_obj *, u8 *byte, u32 indexPort, u32 dataPort, u32 offset);
DLL.cmos_obj_read_byte.argtypes = [ ctypes.POINTER(_CmosAccess), ctypes.POINTER(ctypes.c_uint8), ctypes.c_uint32, ctypes.c_uint32, ctypes.c_uint32 ]
DLL.cmos_obj_read_byte.restype = ctypes.c_int
DLL.cmos_obj_read_byte.errcheck = errorOnNegativeFN(_strerror)

#int    cmos_obj_write_byte(const struct cmos_access_obj *, u8 byte,  u32 indexPort, u32 dataPort, u32 offset);
DLL.cmos_obj_write_byte.argtypes = [ ctypes.POINTER(_CmosAccess), ctypes.c_uint8, ctypes.c_uint32, ctypes.c_uint32, ctypes.c_uint32 ]
DLL.cmos_obj_write_byte.restype = ctypes.c_int
DLL.cmos_obj_write_byte.errcheck = errorOnNegativeFN(_strerror)

#// useful for checksums, etc
#typedef int (*cmos_write_callback)(const struct cmos_access_obj *, bool, void *);
WRITE_CALLBACK = ctypes.CFUNCTYPE(ctypes.c_int, ctypes.POINTER(_CmosAccess), ctypes.c_bool, ctypes.c_void_p)
FREE_CALLBACK = ctypes.CFUNCTYPE(None, ctypes.c_void_p)

#void cmos_obj_register_write_callback(struct cmos_access_obj *, cmos_write_callback, void *, void (*destruct)(void *));
DLL.cmos_obj_register_write_callback.argtypes = [ ctypes.POINTER(_CmosAccess), WRITE_CALLBACK, ctypes.c_void_p, FREE_CALLBACK ]
DLL.cmos_obj_register_write_callback.restype = None

#int cmos_obj_run_callbacks(const struct cmos_access_obj *m, bool do_update);





# vim:tw=0:expandtab:autoindent:tabstop=4:shiftwidth=4:filetype=python:

  #############################################################################
  #
  # Copyright (c) 2005 Dell Computer Corporation
  # Dual Licenced under GNU GPL and OSL
  #
  #############################################################################
"""
smbios:
    python interface to functions in libsmbios_c  obj/smbios.h
"""

# imports (alphabetical)
import ctypes

from libsmbios_c import libsmbios_c_DLL as DLL
from ._common import errorOnNullPtrFN, errorOnNegativeFN, c_utf8_p
from .trace_decorator import traceLog, getLog, strip_trailing_whitespace

__all__ = ["SmbiosTable", "SMBIOS_DEFAULTS", "SMBIOS_GET_SINGLETON", "SMBIOS_GET_NEW", "SMBIOS_UNIT_TEST_MODE"]

SMBIOS_DEFAULTS      =0x0000
SMBIOS_GET_SINGLETON =0x0001
SMBIOS_GET_NEW       =0x0002
SMBIOS_UNIT_TEST_MODE=0x0004

class TableParseError(Exception): pass

class SmbiosStructure(ctypes.Structure):
    @traceLog()
    @strip_trailing_whitespace()
    def getString(self, off):
        return DLL.smbios_struct_get_string_from_offset(self, off)

    @traceLog()
    @strip_trailing_whitespace()
    def getStringNumber(self, num):
        return DLL.smbios_struct_get_string_number(self, num)

    @traceLog()
    def getType(self):
        return DLL.smbios_struct_get_type(self)

    @traceLog()
    def getLength(self):
        return DLL.smbios_struct_get_length(self)

    @traceLog()
    def getHandle(self):
        return DLL.smbios_struct_get_handle(self)

    # use struct module to pull data out
    @traceLog()
    def getData(self, offset, len):
        buf = ctypes.create_string_buffer(len)
        DLL.smbios_struct_get_data(self, buf, offset, len)
        return buf.raw

@traceLog()
def SmbiosTable(flags=SMBIOS_GET_SINGLETON, *factory_args):
    if flags & SMBIOS_GET_SINGLETON:
        if _SmbiosTable._instance is None:
            _SmbiosTable._instance = _SmbiosTable( flags, *factory_args)
        return _SmbiosTable._instance
    else:
        return _SmbiosTable( flags, *factory_args)

class _SmbiosTable(ctypes.Structure):
    _instance = None

    @traceLog()
    def __init__(self, *args):
        self._tableobj = None
        self._tableobj = DLL.smbios_table_factory(*args)

    def __del__(self):
        if self._tableobj is not None:
            DLL.smbios_table_free(self._tableobj)

    @traceLog()
    def __iter__(self):
        cur = ctypes.POINTER(SmbiosStructure)()
        while 1:
            cur =DLL.smbios_table_get_next_struct( self._tableobj, cur )
            if bool(cur):
                yield cur.contents
            else:
                raise StopIteration(_("hit end of table."))

    @traceLog()
    def iterByType(self, t):
        cur = ctypes.POINTER(SmbiosStructure)()
        while 1:
            cur =DLL.smbios_table_get_next_struct( self._tableobj, cur )
            if bool(cur):
                if cur.contents.getType() == t:
                    yield cur.contents
            else:
                raise StopIteration(_("hit end of table."))

    @traceLog()
    def getStructureByHandle(self, handle):
        cur = ctypes.POINTER(SmbiosStructure)()
        cur =DLL.smbios_table_get_next_struct_by_handle( self._tableobj, cur, handle )
        if not bool(cur):
            raise IndexError(_("No SMBIOS structure found with handle %s") % handle)
        return cur.contents

    @traceLog()
    def getStructureByType(self, t):
        cur = ctypes.POINTER(SmbiosStructure)()
        cur =DLL.smbios_table_get_next_struct_by_type( self._tableobj, cur, t )
        if not bool(cur):
            raise IndexError(_("No SMBIOS structure found with type %s") % t)
        return cur.contents

    __getitem__ = getStructureByType

#// format error string
#const char *smbios_table_strerror(const struct smbios_table *m);
# define strerror first so we can use it in error checking other functions.
DLL.smbios_table_strerror.argtypes = [ ctypes.POINTER(_SmbiosTable) ]
DLL.smbios_table_strerror.restype = c_utf8_p
@traceLog()
def _strerror(obj):
    return DLL.smbios_table_strerror(obj)

#struct smbios_table *smbios_table_factory(int flags, ...);
# dont define argtypes because this is a varargs function...
#DLL.smbios_table_factory.argtypes = [ctypes.c_int, ]
DLL.smbios_table_factory.restype = ctypes.POINTER(_SmbiosTable)
DLL.smbios_table_factory.errcheck = errorOnNullPtrFN(lambda r,f,a: TableParseError(_strerror(r)))

#void   smbios_table_free(struct smbios_table *);
DLL.smbios_table_free.argtypes = [ ctypes.POINTER(_SmbiosTable) ]
DLL.smbios_table_free.restype = None

#struct smbios_struct *smbios_table_get_next_struct(const struct smbios_table *, const struct smbios_struct *cur);
DLL.smbios_table_get_next_struct.argtypes = [ ctypes.POINTER(_SmbiosTable), ctypes.POINTER(SmbiosStructure) ]
DLL.smbios_table_get_next_struct.restype = ctypes.POINTER(SmbiosStructure)

#struct smbios_struct *smbios_table_get_next_struct_by_type(const struct smbios_table *, const struct smbios_struct *cur);
DLL.smbios_table_get_next_struct_by_type.argtypes = [ ctypes.POINTER(_SmbiosTable), ctypes.POINTER(SmbiosStructure), ctypes.c_uint8 ]
DLL.smbios_table_get_next_struct_by_type.restype = ctypes.POINTER(SmbiosStructure)

#struct smbios_struct *smbios_table_get_next_struct_by_handle(const struct smbios_table *, const struct smbios_struct *cur);
DLL.smbios_table_get_next_struct_by_handle.argtypes = [ ctypes.POINTER(_SmbiosTable), ctypes.POINTER(SmbiosStructure), ctypes.c_uint16 ]
DLL.smbios_table_get_next_struct_by_handle.restype = ctypes.POINTER(SmbiosStructure)

#u8 DLL_SPEC smbios_struct_get_type(const struct smbios_struct *);
DLL.smbios_struct_get_type.argtypes = [ ctypes.POINTER(SmbiosStructure) ]
DLL.smbios_struct_get_type.restype = ctypes.c_uint8

#u8 DLL_SPEC smbios_struct_get_length(const struct smbios_struct *);
DLL.smbios_struct_get_length.argtypes = [ ctypes.POINTER(SmbiosStructure) ]
DLL.smbios_struct_get_length.restype = ctypes.c_uint8

#u16 DLL_SPEC smbios_struct_get_handle(const struct smbios_struct *);
DLL.smbios_struct_get_handle.argtypes = [ ctypes.POINTER(SmbiosStructure) ]
DLL.smbios_struct_get_handle.restype = ctypes.c_uint16

#const char * DLL_SPEC smbios_struct_get_string_from_offset(const struct smbios_struct *s, u8 offset);
DLL.smbios_struct_get_string_from_offset.argtypes = [ ctypes.POINTER(SmbiosStructure), ctypes.c_uint8 ]
DLL.smbios_struct_get_string_from_offset.restype = c_utf8_p
DLL.smbios_struct_get_string_from_offset.errcheck = errorOnNullPtrFN(lambda r,f,a: IndexError)

#const char * DLL_SPEC smbios_struct_get_string_number(const struct smbios_struct *s, u8 which);
DLL.smbios_struct_get_string_number.argtypes = [ ctypes.POINTER(SmbiosStructure), ctypes.c_uint8 ]
DLL.smbios_struct_get_string_number.restype = c_utf8_p
DLL.smbios_struct_get_string_number.errcheck = errorOnNullPtrFN(lambda r,f,a: IndexError)

#int DLL_SPEC smbios_struct_get_data(const struct smbios_struct *s, void *dest, u8 offset, size_t len);
DLL.smbios_struct_get_data.argtypes = [ ctypes.POINTER(SmbiosStructure), ctypes.c_void_p, ctypes.c_uint8, ctypes.c_size_t ]
DLL.smbios_struct_get_data.restype = ctypes.c_int
DLL.smbios_struct_get_data.errcheck = errorOnNegativeFN(lambda r,f,a: IndexError)
